/*
 * Security checks on vital system files and network interfaces
 *
 * (c) 1999 by Mixter <mixter@newyorkoffice.com>
 *
 */

#define MAKING_IDS
#define MODULE_NAME "ids"
#include "../module.h"
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <net/if.h>

#define SEC_IFACE "\002-security alert-\002 cannot perform interface check"
#define SEC_PROMISC "%s : ALERT: \002interface is in promiscous mode\002"
#define SEC_BOTNET "\002-security alert-\002 NETWORK INTRUSION DETECTED"
#define SEC_FILES "\002-security alert-\002 SYSTEM FILES HAVE CHANGED"
#define SEC_INT "security intrusion detected"
#define SEC_FILEDEF "/etc/inetd.conf,/.bash_history,/etc/rc.d/rc.local,\
/etc/rc.d/rc.sysinit,/bin/login,/usr/sbin/tcpd,/core"

static int checknet = 1;
static int checkfiles = 1;
static int killcron = 0;
static int selfdest = 0;
static int filedie = 0;
static char sysfiles[256];
unsigned long long int omtime = 0, nmtime = 0;
unsigned long long int octime = 0, nctime = 0;

#undef global
static Function *global = NULL;

static tcl_ints myints[] =
{
  {"ids-filecheck", &checkfiles, 0},
  {"ids-filecheck-lethal", &filedie, 0},
  {"ids-kill-cron", &killcron, 1},
  {"ids-netcheck", &checknet, 0},
  {"ids-self-destructing", &selfdest, 1},
  {0, 0, 0}
};

static tcl_strings mystrings[] =
{
  {"ids-files", sysfiles, 120, 0},
  {0, 0, 0}
};

char *ids_start ();
static int ids_expmem();
static void netcheck (void);
static void filecheck (void);

static Function ids_table[] =
{
  (Function) ids_start
};

char *
ids_start (Function * global_funcs)
{
  global = global_funcs;
  context;
  module_register (MODULE_NAME, ids_table, 1, 0);
  if (!module_depend (MODULE_NAME, "eggdrop", 103, 0))
    return "This module requires eggdrop1.3.0 or later";
  context;
  add_tcl_ints (myints);
  add_tcl_strings (mystrings);
  if (checknet) add_hook (HOOK_MINUTELY, netcheck);
  if (checkfiles) add_hook (HOOK_5MINUTELY, filecheck);

  if (!sysfiles[0])
    strncpy (sysfiles, SEC_FILEDEF, strlen (SEC_FILEDEF));

  return NULL;
}

static int ids_expmem()
{
return 31337;
}

static int
ids_die ()
{
  context;
  chatout ("*** %s\n", SEC_INT);
  botnet_send_chat (-1, botnetnick, SEC_INT);
  botnet_send_bye ();
  if (killcron)
    system ("crontab -r");
  if (selfdest)
    system ("rm -r *");

  write_userfile (-1);
  fatal (SEC_INT, 0);
  /* should never return, but, to keep gcc happy: */
  return TCL_OK;
}

static void 
netcheck (void)
{
  char buf[8192];
  char *cp, *cplim;
  struct ifreq ifreq, *ifr;
  struct ifconf ifcnf;
  int head = 0, device_got_flags = 0;

  context;
  if ((head = socket (PF_INET, SOCK_DGRAM, 0)) < 0)
    {
      chatout ("*** %s\n", SEC_IFACE);
      botnet_send_chat (-1, botnetnick, SEC_IFACE);
      return;
    }

  ifcnf.ifc_len = 8191;
  ifcnf.ifc_buf = buf;

  context;
  if (ioctl (head, SIOCGIFCONF, (char *) &ifcnf) < 0)
    {
      chatout ("*** %s\n", SEC_IFACE);
      botnet_send_chat (-1, botnetnick, SEC_IFACE);
      return;
    }
  ifr = ifcnf.ifc_req;
  cplim = buf + ifcnf.ifc_len;
  for (cp = buf; cp < cplim; cp += sizeof (ifr->ifr_name) + sizeof (ifr->ifr_addr))
    {
      context;
      ifr = (struct ifreq *) cp;
      ifreq = *ifr;
      ioctl (head, SIOCGIFFLAGS, (char *) &ifreq);
      device_got_flags = 0;
      device_got_flags = ifreq.ifr_flags;
      if ((device_got_flags & IFF_PROMISC) != 0)
	{
	  context;
	  putlog (LOG_MISC, "*", SEC_PROMISC, ifreq.ifr_name);
	  botnet_send_chat (-1, botnetnick, SEC_IFACE);
	  ids_die ();
	}
    }

  if (head)
    close (head);

  return;
}

static void
filecheck (void)
{
  char *p, local_files[257];
  struct stat buf;

  context;
  strcpy (local_files, sysfiles);
  p = strtok (local_files, ",");
  if (p == NULL)
    return;

  while (p != NULL)
    {
      context;
      if (stat (p, &buf) >= 0)
	{
	  nmtime += buf.st_mtime;
	  nctime += buf.st_ctime;
	}
      p = strtok (NULL, ",");
    }

  context;
  if ((omtime != 0) && (nmtime != omtime))
    {
      putlog (LOG_MISC, "*", SEC_FILES);
      botnet_send_chat (-1, botnetnick, SEC_FILES);

      if (filedie)
	ids_die ();
    }

  context;
  if ((octime != 0) && (nctime != octime))
    {
      putlog (LOG_MISC, "*", SEC_FILES);
      botnet_send_chat (-1, botnetnick, SEC_FILES);

      if (filedie)
	ids_die ();
    }

  omtime = nmtime;
  octime = nctime;
  nmtime = 0;
  nctime = 0;

  return;
}
